#include "Config.h"

/*

  {
    "dirs" : {
        "app" : ".",
        "pads" : "pads"
    },
    "autoconnect" : true,
	"clear" : true,
	"server" : {
		"type" : "stdin" // "stdin", "af_unix", "af_inet"
		["address" : PATH or DOMAIN_NAME,]
		["port" : PORT,]
		["keep-alive" : false]
	},

  }


  */
std::string Config::m_defaut_config="{\
                                    \"dirs\" : {\
                                        \"app\" : \".\",\
                                        \"pads\" : \"pads\"\
                                    },\
                                    \"autoconnect\" : true,\
									\"clear\" : true,\
									\"server\" : {\
										\"type\" : \"stdin\",\
										\"keep-alive\" :false,\
									},\
                                  }";

Config* Config::m_instance=NULL;

#include <fstream>
#include <sstream>
#include <iostream>
#include "utils.h"
Config::Config(Args& args)
{
    _init(args.config_file);
    if(args.override_app_dir!="") _set("dirs.app", Json::Value(args.override_app_dir));

    if(args.override_pad_dir!=""){
        Json::Value v(args.override_pad_dir);
        _set("dirs.pads", v);
    }

	if(args.use_pipe){
		_set("server.type", "af_unix");
		_set("server.address", args.use_pipe_filename);
		_set("server.port", -1);
	}else if(args.use_socket){
		_set("server.type", "inet");
		_set("server.address", "localhost");
		_set("server.port", args.use_socket_port);
	}else if(args.use_stdin){
		_set("server.type", "stdin");
		_set("server.address", "");
		_set("server.port", -1);
	}

	if(args.keep_alive){
		_set("server.keep-alive", args.keep_alive==1);
	}

	if(0)
	{
		std::cerr << "Affichage de la config"  << "\n";
		std::cerr << m_root << "\n";
	}
}

void Config::check() {
	if(!Fs::is_dir(Config::APP_DIR())){
		std::cerr << "Le dossier de l'application '"<< Config::APP_DIR() <<"' est inaccessible\nFermeture...\n";
		exit(-1);
	}
	if(!Fs::is_dir(Config::PAD_DIR())){
		std::cerr << "Le dossier de pad '"<< Config::PAD_DIR() <<"' est inaccessible\nFermeture...\n";
		exit(-1);
	}
}

void Config::init(Args &args){
    m_instance=new Config(args);
	m_instance->check();
}

void Config::_init(const std::string& file)
{
    Json::Value v;
    if(file=="")
    {
        std::istringstream s(m_defaut_config);
        std::cerr << "Lecture de la config par défaut\n";
        s >> v;
    }
    else{
        std::ifstream fd(file);
        if(fd.is_open())
        {
            try{
                fd >> v;
            }catch(...){
                std::cerr << "Erreur: problème dans le fichier de config '"<< file <<"'\n";
                fd.close();
                return _init();
            }

            fd.close();
        }
        else
        {
            std::cerr << "Impossible de lire le fichier de config '"<<file<<"'\n";
            return _init();
        }
    }
    m_root=v;


}

Json::Value Config::_get(const std::string& _path)
{
    const char* path = _path.c_str();
    char* curr = (char*) path;

    Json::Value v=m_root;
    while(*curr){
        std::string tmp;
        while(*curr && *curr!='.') tmp+=(*curr++);
        if(!v.isObject() || !v.isMember(tmp)){
			THROW(JsonPathNotFoundError, "L'option '"<<_path<<"' est inconnu ('"<<tmp<<"' est introuvable)");
        }
        v=v[tmp];
        if(*curr) curr++;
    }
    return v;
}


void Config::_set(const std::string& _path, Json::Value val)
{
    const char* path = _path.c_str();
    char* curr = (char*) path;

    Json::Value* v=&m_root;
    while(*curr){
        std::string tmp;
        while(*curr && *curr!='.') tmp+=(*curr++);
		if(!v->isObject()){
			(*v)=Json::ValueType::objectValue;
        }
		if(!v->isMember(tmp)){
			(*v)[tmp]=Json::ValueType::objectValue;
		}
        v=&((*v)[tmp]);
        if(*curr) curr++;
    }
    *v=val;
}


